
from .spellcheck import SpellcheckService, SpellcheckResource


def init_app(app):
    endpoint_name = 'spellcheck'
    service = SpellcheckService(endpoint_name, backend=None)
    SpellcheckResource(endpoint_name, app=app, service=service)



#!/usr/bin/env python
# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2016 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import superdesk
from superdesk.utils import ListCursor
from flask import current_app as app


class AllowedValuesResource(superdesk.Resource):
    """Resource allowing apps to set allowed values which can be used by client to filter certain options."""

    resource_methods = ['GET']
    item_methods = []


class AllowedValuesService(superdesk.Service):

    def get(self, req, lookup):
        allowed = []
        for resource, config in app.config.get('DOMAIN', {}).items():
            for field, field_config in config.get('schema', {}).items():
                if field_config.get('allowed'):
                    allowed.append({
                        '_id': '{}.{}'.format(resource, field),
                        'items': [str(item) for item in field_config['allowed']],
                    })
        return ListCursor(allowed)


def init_app(app):
    AllowedValuesResource('allowed_values', app, AllowedValuesService())



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014, 2015 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from bson import ObjectId
from nose.tools import assert_raises

from apps.publish import init_app
from superdesk.errors import PublishQueueError
from superdesk.publish import SUBSCRIBER_TYPES
from superdesk.publish.publish_service import PublishService
from superdesk.tests import TestCase


class PublishServiceTests(TestCase):
    queue_items = [{"_id": "571075791d41c81e204c5c8c",
                    "destination": {"name": "NITF", "delivery_type": "ftp", "format": "nitf", "config": {}},
                    "subscriber_id": "1",
                    "state": "in-progress",
                    "item_id": 1,
                    "formatted_item": ''
                    }]

    subscribers = [{"_id": "1", "name": "Test", "subscriber_type": SUBSCRIBER_TYPES.WIRE, "media_type": "media",
                    "is_active": True, "sequence_num_settings": {"max": 10, "min": 1},
                    "critical_errors": {"9004": True},
                    "destinations": [{"name": "NITF", "delivery_type": "ftp", "format": "nitf", "config": {}}]
                    }]

    def setUp(self):
        with self.app.app_context():
            self.app.data.insert('subscribers', self.subscribers)
            self.queue_items[0]['_id'] = ObjectId(self.queue_items[0]['_id'])
            self.app.data.insert('publish_queue', self.queue_items)

            init_app(self.app)

    def test_close_subscriber_doesnt_close(self):
        with self.app.app_context():
            subscriber = self.app.data.find('subscribers', None, None)[0]
            self.assertTrue(subscriber.get('is_active'))

            PublishService().close_transmitter(subscriber, PublishQueueError.unknown_format_error())
            subscriber = self.app.data.find('subscribers', None, None)[0]
            self.assertTrue(subscriber.get('is_active'))

    def test_close_subscriber_does_close(self):
        with self.app.app_context():
            subscriber = self.app.data.find('subscribers', None, None)[0]
            self.assertTrue(subscriber.get('is_active'))

            PublishService().close_transmitter(subscriber, PublishQueueError.bad_schedule_error())
            subscriber = self.app.data.find('subscribers', None, None)[0]
            self.assertFalse(subscriber.get('is_active'))

    def test_transmit_closes_subscriber(self):
        def mock_transmit(*args):
            raise PublishQueueError.bad_schedule_error()

        with self.app.app_context():
            subscriber = self.app.data.find('subscribers', None, None)[0]

            publish_service = PublishService()
            publish_service._transmit = mock_transmit

            with assert_raises(PublishQueueError):
                publish_service.transmit(self.queue_items[0])

            subscriber = self.app.data.find('subscribers', None, None)[0]
            self.assertFalse(subscriber.get('is_active'))
            self.assertIsNotNone(subscriber.get('last_closed'))



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from superdesk.resource import Resource
from content_api import MONGO_PREFIX


class CompaniesResource(Resource):
    """
    Company schema
    """

    schema = {
        'name': {
            'type': 'string',
            'unique': True,
            'required': True
        },
        'sd_subscriber_id': {
            'type': 'string'
        },
        'is_enabled': {
            'type': 'boolean',
            'default': True
        },
        'contact_name': {
            'type': 'string'
        },
        'phone': {
            'type': 'string'
        },
        'country': {
            'type': 'string'
        }
    }
    datasource = {
        'source': 'companies',
        'default_sort': [('name', 1)]
    }
    item_methods = ['GET', 'PATCH', 'PUT']
    resource_methods = ['GET', 'POST']
    mongo_prefix = MONGO_PREFIX



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from unittest import mock
from datetime import timedelta

from apps.publish import init_app
from apps.publish.enqueue import enqueue_items, get_published_items
from superdesk import config
from superdesk.publish.publish_content import get_queue_items
from superdesk.tests import TestCase
from superdesk.utc import utcnow


class PublishContentTests(TestCase):
    queue_items = [{"_id": 1,
                    "destination": {
                        "delivery_type": "ftp",
                        "config": {},
                        "name": "destination1"
                    },
                    "_etag": "f28b9af64f169072fb171ec7f316fc03d5826d6b",
                    "subscriber_id": "552ba73f1d41c8437971613e",
                    "state": "pending",
                    "_created": "2015-04-17T13:15:20.000Z",
                    "_updated": "2015-04-20T05:04:25.000Z",
                    "item_id": 1
                    },
                   {
                       "_id": 2,
                       "destination": {
                           "delivery_type": "ftp",
                           "config": {},
                           "name": "destination1"
                       },
                       "_etag": "f28b9af64f169072fb171ec7f316fc03d5826d6b",
                       "subscriber_id": "552ba73f1d41c8437971613e",
                       "state": "pending",
                       "_created": "2015-04-17T13:15:20.000Z",
                       "_updated": "2015-04-20T05:04:25.000Z",
                       "item_id": 1,
                       "publish_schedule": utcnow() + timedelta(minutes=10)},
                   {
                       "_id": 3,
                       "destination": {
                           "delivery_type": "ftp",
                           "config": {},
                           "name": "destination1"
                       },
                       "_etag": "f28b9af64f169072fb171ec7f316fc03d5826d6b",
                       "subscriber_id": "552ba73f1d41c8437971613e",
                       "state": "pending",
                       "_created": "2015-04-17T13:15:20.000Z",
                       "_updated": "2015-04-20T05:04:25.000Z",
                       "item_id": '2',
                       "publish_schedule": "2015-04-20T05:04:25.000Z"},
                   {
                       "_id": 4,
                       "destination": {
                           "delivery_type": "content_api",
                           "format": "ninjs",
                           "config": {},
                           "name": "destination1"
                       },
                       "_etag": "f28b9af64f169072fb171ec7f316fc03d5826d6b",
                       "subscriber_id": "552ba73f1d41c8437971613e",
                       "state": "success",
                       "_created": "2015-04-17T13:15:20.000Z",
                       "_updated": "2015-04-20T05:04:25.000Z",
                       "item_id": '2'}]

    published_items = [
        {
            "_id": 1,
            "item_id": "1",
            "_created": utcnow(),
            "_updated": utcnow(),
            "queue_state": "pending",
            "state": "published"
        },
        {
            "_id": 2,
            "item_id": "2",
            "_created": utcnow(),
            "_updated": utcnow(),
            "queue_state": "pending",
            "state": "scheduled",
            "publish_schedule": utcnow() - timedelta(minutes=5),
            "schedule_settings": {
                "utc_publish_schedule": utcnow() - timedelta(minutes=5),
                "timezone": "UTC",
                "utc_embargo": None
            }
        },
        {
            "_id": 3,
            "item_id": "3",
            "_created": utcnow(),
            "_updated": utcnow(),
            "queue_state": "pending",
            "state": "scheduled",
            "publish_schedule": utcnow() + timedelta(minutes=60),
            "schedule_settings": {
                "utc_publish_schedule": utcnow() + timedelta(minutes=60),
                "timezone": "UTC",
                "utc_embargo": None
            }
        }

    ]

    def setUp(self):
        with self.app.app_context():
            init_app(self.app)

    def test_queue_items(self):
        with self.app.app_context():
            self.app.data.insert('publish_queue', self.queue_items)
            items = get_queue_items()
            self.assertEqual(3, items.count())
            ids = [item[config.ID_FIELD] for item in items]
            self.assertNotIn(4, ids)

    @mock.patch('apps.publish.enqueue.enqueue_item')
    def test_enqueue_item_not_scheduled(self, *mocks):
        fake_enqueue_item = mocks[0]
        queue_items = [
            {'_id': '1', 'item_id': 'item_1', 'queue_state': 'pending',
             'state': 'published'}
        ]
        enqueue_items(queue_items)
        fake_enqueue_item.assert_called_with(queue_items[0])

    def test_get_enqueue_items(self):
        self.app.data.insert('published', self.published_items)
        items = get_published_items()
        self.assertEqual(2, len(items))
        ids = [item[config.ID_FIELD] for item in items]
        self.assertNotIn(3, ids)




from superdesk import Resource, Service
from superdesk.utils import ListCursor
from superdesk.errors import AlreadyExistsError


registered_search_providers = {}
allowed_search_providers = []


def register_search_provider(name, fetch_endpoint=None, provider_class=None, label=None):
    """Register a Search Provider with the given name and fetch_endpoint.

    Both have to be unique and if not raises AlreadyExistsError.
    The fetch_endpoint is used by clients to fetch the article from the Search Provider.

    :param name: Search Provider Name
    :type name: str
    :param fetch_endpoint: relative url to /api
    :type fetch_endpoint: str
    :param provider_class: provider implementation
    :type provider: superdesk.SearchProvider
    :param label: label to use (None to use provider_class.label or name in this order)
    :type label: str
    :raises: AlreadyExistsError - if a search has been registered with either name or fetch_endpoint.
    """
    if fetch_endpoint is not None and not isinstance(fetch_endpoint, str):
        raise ValueError("fetch_enpoint must be a string")
    if name in registered_search_providers:
        raise AlreadyExistsError("A Search Provider with name: {} already exists".format(name))

    if not ((fetch_endpoint and not provider_class) or (not fetch_endpoint and provider_class)):
        raise ValueError('You have to specify either fetch_endpoint or provider_class.')

    provider_data = {}

    if fetch_endpoint:
        existing_endpoints = {d['endpoint'] for d in registered_search_providers.values() if 'endpoint' in d}
        if fetch_endpoint in existing_endpoints:
            raise AlreadyExistsError("A Search Provider for the fetch endpoint: {} exists with name: {}"
                                     .format(fetch_endpoint, registered_search_providers[name]))
        provider_data['endpoint'] = fetch_endpoint
    else:
        provider_data['class'] = provider_class

    if label is not None:
        provider_data['label'] = label
    elif provider_class is not None and hasattr(provider_class, 'label') and provider_class.label:
        provider_data['label'] = provider_class.label
    else:
        provider_data['label'] = name

    provider_data = registered_search_providers[name] = provider_data

    allowed_search_providers.append(name)


class SearchProviderAllowedResource(Resource):
    resource_methods = ['GET']
    item_methods = []


class SearchProviderAllowedService(Service):

    def get(self, req, lookup):
        def provider(provider_id):
            provider_data = registered_search_providers[provider_id]
            return {
                'search_provider': provider_id,
                'label': provider_data['label']
            }

        return ListCursor(
            [provider(_id) for _id in registered_search_providers]
        )



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from .anpa import ANPAFeedParser
from superdesk.io.registry import register_feed_parser
from superdesk.io.iptc import subject_codes
from flask import current_app as app
from apps.archive.common import format_dateline_to_locmmmddsrc
from superdesk.utc import get_date
from superdesk import get_resource_service
from superdesk.etree import parse_html
import logging
import re

logger = logging.getLogger("AP_ANPAFeedParser")


class AP_ANPAFeedParser(ANPAFeedParser):
    """
    Feed parser for AP supplied ANPA, maps category codes and maps the prefix on some sluglines to subject codes
    """

    NAME = 'ap_anpa1312'

    label = 'AP ANPA Parser'

    slug_code_map = {  # Basketball
                       'BKC-': '15008000', 'BKL-': '15008000', 'BKN-': '15008000',
                       'BKW-': '15008000', 'BKO-': '15008000',
                       # Baseball
                       'BBA-': '15007000', 'BBC-': '15007000', 'BBM-': '15007000', 'BBN-': '15007000',
                       'BBO-': '15007000', 'BBY-': '15007000',
                       # Boxing
                       'BOX-': '15014000',
                       # Motor racing
                       'CAR-': '15039000',
                       # Cycling
                       'CYC-': '15019000',
                       # American Football
                       'FBL-': '15003000', 'FBN-': '15003000', 'FBO-': '15003000',
                       # Figure skating
                       'FIG-': '15025000',
                       # Golf
                       'GLF-': '15027000',
                       # Ice Hockey
                       'HKN-': '15031000', 'HKO-': '15031000',
                       # Horse Racing
                       'RAC-': '15030000',
                       # Soccer
                       'SOC-': '15054000',
                       # Tennis
                       'TEN-': '15065000',
                       # Cricket
                       'CRI-': '15017000',
                       # Rugby league
                       'RGL-': '15048000'}

    def parse(self, file_path, provider=None):
        item = super().parse(file_path, provider)
        self.ap_derive_dateline(item)
        self.map_category_codes(item)
        self.map_sluglines_to_subjects(item)
        if 'headline' not in item:
            self._fix_headline(item)
        return item

    def map_category_codes(self, item):
        """Map the category code that has been received to a more palatable value

        :param item:
        :return:
        """
        for category in item.get('anpa_category', []):
            if category.get('qcode').lower() in ('a', 'p', 'w', 'n'):
                category['qcode'] = 'i'
            elif category.get('qcode').lower() == 'r':
                category['qcode'] = 'v'
            elif category.get('qcode').lower() == 'z':
                category['qcode'] = 's'
            else:
                # check if the category is defined in the system
                category_map = get_resource_service('vocabularies').find_one(req=None, _id='categories')
                if category_map:
                    if not next((code for code in category_map['items'] if
                                 code['qcode'].lower() == category['qcode'].lower() and code['is_active']), None):
                        category['qcode'] = 'i'
                else:
                    category['qcode'] = 'i'

    def map_sluglines_to_subjects(self, item):
        """The first few characters of the slugline may match AP supplimetal categories this is used to set the subject code.

        :param item:
        :return:
        """
        if len(item.get('slugline', '')) > 4:
            qcode = self.slug_code_map.get(item['slugline'][:4])
            if qcode:
                try:
                    item['subject'] = []
                    item['subject'].append({'qcode': qcode, 'name': subject_codes[qcode]})
                except KeyError:
                    logger.debug("Subject code '%s' not found" % qcode)

    def ap_derive_dateline(self, item):
        """This function looks for a dateline in the article body an uses that.

        :param item:
        :return: item populated with a dateline
        """
        try:
            html = item.get('body_html')
            if html:
                parsed = parse_html(html, content='html')
                for par in parsed.xpath('/html/div/child::*'):
                    if not par.text:
                        continue
                    city, source, the_rest = par.text.partition(' (AP) _ ')
                    if source:
                        # sometimes the city is followed by a comma and either a date or a state
                        city = city.split(',')[0]
                        if any(char.isdigit() for char in city):
                            return
                        cities = app.locators.find_cities()
                        located = [c for c in cities if c['city'].lower() == city.lower()]
                        item.setdefault('dateline', {})
                        item['dateline']['located'] = located[0] if len(located) == 1 else {'city_code': city,
                                                                                            'city': city,
                                                                                            'tz': 'UTC',
                                                                                            'dateline': 'city'}
                        item['dateline']['source'] = item.get('original_source', 'AP')
                        item['dateline']['text'] = format_dateline_to_locmmmddsrc(item['dateline']['located'],
                                                                                  get_date(item['firstcreated']),
                                                                                  source=item.get('original_source',
                                                                                                  'AP'))
                        break

            return item
        except Exception:
            logging.exception('AP dateline extraction exception')

    def _parse_ednote(self, header_lines, item):
        """
        Attempt to parse the ednote, If no ednote is found by the base class try another pattern
        :param header_lines:
        :param item:
        :return:
        """
        super()._parse_ednote(header_lines, item)
        if not item.get('ednote'):
            for line in header_lines:
                m = re.search("(\(?)Eds: (.*)(\)?)", line)
                if m:
                    item['ednote'] = m.group(0).strip()[:-1].replace('Eds: ', '')
                    break

    def _fix_headline(self, item):
        """
        AP Alerts do not get a healdine parsed out so pick up the first par of the content and put it in the headline
        :param item:
        :return:
        """
        try:
            html = item.get('body_html')
            if html:
                parsed = parse_html(html, content='html')
                pars = parsed.xpath('/html/div/child::*')
                if pars and len(pars) > 0:
                    city, source, the_rest = pars[0].text.partition(' (AP) _ ')
                    if the_rest:
                        item['headline'] = the_rest
                    else:
                        item['headline'] = pars[0].text
        except Exception:
            pass


register_feed_parser(AP_ANPAFeedParser.NAME, AP_ANPAFeedParser())



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import time
import logging
from lxml import etree
from lxml.etree import SubElement
from eve.utils import config
from superdesk.publish.formatters import Formatter
import superdesk
from superdesk.errors import FormatterError
from superdesk.etree import parse_html
from superdesk.metadata.item import ITEM_TYPE, CONTENT_TYPE, EMBARGO, ITEM_STATE, CONTENT_STATE,\
    GUID_FIELD
from superdesk.metadata.packages import GROUP_ID, REFS, RESIDREF, ROLE, ROOT_GROUP
from superdesk.utc import utcnow
from flask import current_app as app
from apps  .archive.common import get_utc_schedule
from superdesk.filemeta import get_filemeta

logger = logging.getLogger(__name__)


class NewsML12Formatter(Formatter):
    """
    NewsML 1.2 Formatter
    """

    XML_ROOT = '<?xml version="1.0"?><!DOCTYPE NewsML SYSTEM "http://www.provider.com/dtd/NewsML_1.2.dtd">'
    now = utcnow()
    string_now = now.strftime('%Y%m%dT%H%M%S+0000')
    newml_content_type = {
        CONTENT_TYPE.PICTURE: 'Photo',
        CONTENT_TYPE.AUDIO: 'Audio',
        CONTENT_TYPE.VIDEO: 'Video',
        CONTENT_TYPE.GRAPHIC: 'Graphic',
        CONTENT_TYPE.COMPOSITE: 'ComplexData',
        CONTENT_TYPE.PREFORMATTED: 'Text',
        CONTENT_TYPE.TEXT: 'Text'
    }

    def format(self, article, subscriber, codes=None):
        """
        Create article in NewsML1.2 format

        :param dict article:
        :param dict subscriber:
        :param list codes:
        :return [(int, str)]: return a List of tuples. A tuple consist of
            publish sequence number and formatted article string.
        :raises FormatterError: if the formatter fails to format an article
        """
        try:
            pub_seq_num = superdesk.get_resource_service('subscribers').generate_sequence_number(subscriber)

            newsml = etree.Element("NewsML")
            SubElement(newsml, "Catalog", {'Href': 'http://www.iptc.org/std/catalog/catalog.IptcMasterCatalog.xml'})
            news_envelope = SubElement(newsml, "NewsEnvelope")
            news_item = SubElement(newsml, "NewsItem")

            self._format_news_envelope(article, news_envelope, pub_seq_num)
            self._format_identification(article, news_item)
            self._format_news_management(article, news_item)
            self._format_news_component(article, news_item)

            return [(pub_seq_num, self.XML_ROOT + etree.tostring(newsml).decode('utf-8'))]
        except Exception as ex:
            raise FormatterError.newml12FormatterError(ex, subscriber)

    def _format_news_envelope(self, article, news_envelope, pub_seq_num):
        """
        Create a NewsEnvelope element

        :param dict article:
        :param element news_envelope:
        :param int pub_seq_num:
        """
        SubElement(news_envelope, 'TransmissionId').text = str(pub_seq_num)
        SubElement(news_envelope, 'DateAndTime').text = self.string_now
        SubElement(news_envelope, 'Priority', {'FormalName': str(article.get('priority', 5))})

    def _format_identification(self, article, news_item):
        """
        Creates the Identification element

        :param dict article:
        :param Element news_item:
        """
        revision = self._process_revision(article)
        identification = SubElement(news_item, "Identification")
        news_identifier = SubElement(identification, "NewsIdentifier")
        date_id = article.get('firstcreated').strftime("%Y%m%d")
        SubElement(news_identifier, 'ProviderId').text = app.config['NEWSML_PROVIDER_ID']
        SubElement(news_identifier, 'DateId').text = date_id
        SubElement(news_identifier, 'NewsItemId').text = article[GUID_FIELD]
        SubElement(news_identifier, 'RevisionId', attrib=revision).text = str(article.get(config.VERSION, ''))
        SubElement(news_identifier, 'PublicIdentifier').text = \
            self._generate_public_identifier(article[config.ID_FIELD], article.get(config.VERSION, ''),
                                             revision.get('Update', ''))
        SubElement(identification, "DateLabel").text = self.now.strftime("%A %d %B %Y")

    def _generate_public_identifier(self, item_id, version, update):
        """
        Create the NewsML public identifier

        :param str item_id: Article Id
        :param str version: Article Version
        :param str update: update
        :return: returns public identifier
        """
        return '%s:%s%s' % (item_id, version, update)

    def _process_revision(self, article):
        """
        Get attributes of RevisionId element.

        :param dict article:
        :return: dict
        """
        revision = {'PreviousRevision': '0', 'Update': 'N'}
        if article.get(ITEM_STATE) in {CONTENT_STATE.CORRECTED, CONTENT_STATE.KILLED}:
            revision['PreviousRevision'] = str(article.get(config.VERSION) - 1)
        return revision

    def _format_news_management(self, article, news_item):
        """
        Create a NewsManagement element

        :param dict article:
        :param Element news_item:
        """
        news_management = SubElement(news_item, "NewsManagement")
        SubElement(news_management, 'NewsItemType', {'FormalName': 'News'})
        SubElement(news_management, 'FirstCreated').text = \
            article['firstcreated'].strftime('%Y%m%dT%H%M%S+0000')
        SubElement(news_management, 'ThisRevisionCreated').text = \
            article['versioncreated'].strftime('%Y%m%dT%H%M%S+0000')

        if article.get(EMBARGO):
            SubElement(news_management, 'Status', {'FormalName': 'Embargoed'})
            status_will_change = SubElement(news_management, 'StatusWillChange')
            SubElement(status_will_change, 'FutureStatus', {'FormalName': article['pubstatus']})
            SubElement(status_will_change, 'DateAndTime').text = \
                get_utc_schedule(article, EMBARGO).isoformat()
        else:
            SubElement(news_management, 'Status', {'FormalName': article['pubstatus']})

        if article.get('urgency'):
            SubElement(news_management, 'Urgency', {'FormalName': str(article['urgency'])})

        if article['state'] == 'corrected':
            SubElement(news_management, 'Instruction', {'FormalName': 'Correction'})
        else:
            SubElement(news_management, 'Instruction', {'FormalName': 'Update'})

    def _format_news_component(self, article, news_item):
        """
        Create a main NewsComponent element

        :param dict article:
        :param Element news_item:
        """
        news_component = SubElement(news_item, "NewsComponent")
        main_news_component = SubElement(news_component, "NewsComponent")
        SubElement(main_news_component, 'Role', {'FormalName': 'Main'})
        self._format_news_lines(article, main_news_component)
        self._format_rights_metadata(article, main_news_component)
        self._format_descriptive_metadata(article, main_news_component)

        for company in article.get('company_codes', []):
            metadata = SubElement(main_news_component, 'Metadata')
            SubElement(metadata, 'MetadataType', attrib={'FormalName': 'Securities Identifier'})
            SubElement(metadata, 'Property', attrib={'FormalName': 'Name', 'Value': company.get('name', '')})
            SubElement(metadata, 'Property', attrib={'FormalName': 'Ticker Symbol', 'Value': company.get('qcode', '')})
            SubElement(metadata, 'Property', attrib={'FormalName': 'Exchange',
                                                     'Value': company.get('security_exchange', '')})

        if article.get(ITEM_TYPE) == CONTENT_TYPE.COMPOSITE:
            self._format_package(article, main_news_component)
        elif article.get(ITEM_TYPE) in {CONTENT_TYPE.TEXT, CONTENT_TYPE.PREFORMATTED, CONTENT_TYPE.COMPOSITE}:
            self._format_abstract(article, main_news_component)
            self._format_body(article, main_news_component)
        elif article.get(ITEM_TYPE) in {CONTENT_TYPE.VIDEO, CONTENT_TYPE.AUDIO, CONTENT_TYPE.PICTURE}:
            self._format_media(article, main_news_component, self.newml_content_type[article.get(ITEM_TYPE)])

    def _format_news_lines(self, article, main_news_component):
        """
        Create a NewsLines element

        :param dict article:
        :param Element main_news_component:
        """
        news_lines = SubElement(main_news_component, "NewsLines")
        if article.get('headline'):
            SubElement(news_lines, 'HeadLine').text = article.get('headline')
        if article.get('byline'):
            SubElement(news_lines, 'ByLine').text = article.get('byline') or ''
        if article.get('dateline', {}).get('text', ''):
            SubElement(news_lines, 'DateLine').text = article.get('dateline', {}).get('text', '')
        SubElement(news_lines, 'CreditLine').text = article.get('original_source', article.get('source', ''))
        if article.get('slugline', ''):
            SubElement(news_lines, 'KeywordLine').text = article.get('slugline', '')
        # TODO: may be we need to create our own catalog for the NewsLineType
        if article.get('ednote'):
            news_line = SubElement(news_lines, 'NewsLine')
            SubElement(news_line, 'NewsLineType', attrib={'FormalName': 'EditorialNote'})
            SubElement(news_line, 'NewsLineText').text = article.get('ednote')

    def _format_rights_metadata(self, article, main_news_component):
        """
        Create a RightsMetadata element

        :param dict article:
        :param Element main_news_component:
        """
        rights = superdesk.get_resource_service('vocabularies').get_rightsinfo(article)

        rights_metadata = SubElement(main_news_component, "RightsMetadata")
        copyright = SubElement(rights_metadata, "Copyright")
        SubElement(copyright, 'CopyrightHolder').text = rights['copyrightholder']
        SubElement(copyright, 'CopyrightDate').text = self.now.strftime("%Y")

        usage_rights = SubElement(rights_metadata, "UsageRights")
        SubElement(usage_rights, 'UsageType').text = rights['copyrightnotice']
        # SubElement(usage_rights, 'Geography').text = article.get('place', article.get('located', ''))
        SubElement(usage_rights, 'RightsHolder').text = article.get('original_source', article.get('source', ''))
        SubElement(usage_rights, 'Limitations').text = rights['usageterms']
        SubElement(usage_rights, 'StartDate').text = self.string_now
        SubElement(usage_rights, 'EndDate').text = self.string_now

    def _format_descriptive_metadata(self, article, main_news_component):
        """
        Create a Descriptive_metadata element

        :param dict article:
        :param Element main_news_component:
        """
        descriptive_metadata = SubElement(main_news_component, "DescriptiveMetadata")
        subject_code = SubElement(descriptive_metadata, "SubjectCode")

        for subject in article.get('subject', []):
            SubElement(subject_code, 'Subject', {'FormalName': subject.get('qcode', '')})

        self._format_dateline(article, descriptive_metadata)
        self._format_place(article, descriptive_metadata)

        if 'anpa_category' in article:
            for category in article.get('anpa_category', []):
                SubElement(descriptive_metadata, 'Property',
                           {'FormalName': 'Category', 'Value': category['qcode']})

    def _format_place(self, article, descriptive_metadata):
        """
        Create Place specific element in the descriptive metadata

        :param dict article:
        :param Element descriptive_metadata:
        """
        if not article.get('place'):
            return

        for place in article.get('place'):
            if place.get('country') or place.get('state') or place.get('world_region'):
                location = SubElement(descriptive_metadata,
                                      'Location', attrib={'HowPresent': 'RelatesTo'})
                if place.get('state'):
                    SubElement(location, 'Property',
                               {'FormalName': 'CountryArea', 'Value': place.get('state')})
                if place.get('country'):
                    SubElement(location, 'Property',
                               {'FormalName': 'Country', 'Value': place.get('country')})
                if place.get('world_region'):
                    SubElement(location, 'Property',
                               {'FormalName': 'WorldRegion', 'Value': place.get('world_region')})

    def _format_dateline(self, article, descriptive_metadata):
        """
        Create the DateLineDate and Location in the descriptive metadata

        :param dict article:
        :param Element news_lines:
        """
        located = article.get('dateline', {}).get('located')
        if located:
            location_elm = SubElement(descriptive_metadata, 'Location', attrib={'HowPresent': 'Origin'})
            if located.get('city'):
                SubElement(location_elm, 'Property',
                           attrib={'FormalName': 'City', 'Value': located.get('city')})
            if located.get('state'):
                SubElement(location_elm, 'Property',
                           attrib={'FormalName': 'CountryArea', 'Value': located.get('state')})
            if located.get('country'):
                SubElement(location_elm, 'Property',
                           attrib={'FormalName': 'Country', 'Value': located.get('country')})

    def _format_abstract(self, article, main_news_component):
        """
        Create an abstract NewsComponent element

        :param dict article:
        :param Element main_news_component:
        """
        abstract_news_component = SubElement(main_news_component, "NewsComponent")
        SubElement(abstract_news_component, 'Role', {'FormalName': 'Abstract'})
        content_item = SubElement(abstract_news_component, "ContentItem")
        SubElement(content_item, 'MediaType', {'FormalName': 'Text'})
        SubElement(content_item, 'Format', {'FormalName': 'Text'})
        abstract = parse_html(article.get('abstract', ''))
        SubElement(content_item, 'DataContent').text = etree.tostring(abstract, encoding="unicode", method="text")

    def _format_body(self, article, main_news_component):
        """
        Create an body text NewsComponent element

        :param dict article:
        :param Element main_news_component:
        """
        body_news_component = SubElement(main_news_component, "NewsComponent")
        SubElement(body_news_component, 'Role', {'FormalName': 'BodyText'})
        content_item = SubElement(body_news_component, "ContentItem")
        SubElement(content_item, 'MediaType', {'FormalName': 'Text'})
        SubElement(content_item, 'Format', {'FormalName': 'NITF'})
        data_content = SubElement(content_item, 'DataContent')
        nitf = SubElement(data_content, 'nitf')
        body = SubElement(nitf, 'body')
        body_content = SubElement(body, 'body.content')
        self.map_html_to_xml(body_content, self.append_body_footer(article))

    def _format_description(self, article, main_news_component):
        """
        Create an description NewsComponent element

        :param dict article:
        :param Element main_news_component:
        """
        desc_news_component = SubElement(main_news_component, "NewsComponent")
        SubElement(desc_news_component, 'Role', {'FormalName': 'Description'})
        content_item = SubElement(desc_news_component, "ContentItem")
        SubElement(content_item, 'MediaType', {'FormalName': 'Text'})
        SubElement(content_item, 'Format', {'FormalName': 'Text'})
        SubElement(content_item, 'DataContent').text = self.append_body_footer(article)

    def _format_media(self, article, main_news_component, media_type):
        """
        Create an NewsComponent element related to media

        :param dict article:
        :param Element main_news_component:
        """
        media_news_component = SubElement(main_news_component, "NewsComponent")
        SubElement(media_news_component, 'Role', {'FormalName': media_type})
        for rendition, value in article.get('renditions').items():
            news_component = SubElement(media_news_component, "NewsComponent")
            SubElement(news_component, 'Role', {'FormalName': rendition})
            content_item = SubElement(news_component, "ContentItem",
                                      attrib={'Href': value.get('href', '')})
            SubElement(content_item, 'MediaType', {'FormalName': media_type})
            SubElement(content_item, 'Format', {'FormalName': value.get('mimetype', '')})
            characteristics = SubElement(content_item, 'Characteristics')
            if rendition == 'original' and get_filemeta(article, 'length'):
                SubElement(characteristics, 'SizeInBytes').text = str(get_filemeta(article, 'length'))
            if article.get(ITEM_TYPE) == CONTENT_TYPE.PICTURE:
                if value.get('width'):
                    SubElement(characteristics, 'Property',
                               attrib={'FormalName': 'Width', 'Value': str(value.get('width'))})
                if value.get('height'):
                    SubElement(characteristics, 'Property',
                               attrib={'FormalName': 'Height', 'Value': str(value.get('height'))})
            elif article.get(ITEM_TYPE) in {CONTENT_TYPE.VIDEO, CONTENT_TYPE.AUDIO}:
                if get_filemeta(article, 'width'):
                    SubElement(characteristics, 'Property',
                               attrib={'FormalName': 'Width',
                                       'Value': str(get_filemeta(article, 'width'))})
                if get_filemeta(article, 'height'):
                    SubElement(characteristics, 'Property',
                               attrib={'FormalName': 'Height',
                                       'Value': str(get_filemeta(article, 'height'))})

                duration = self._get_total_duration(get_filemeta(article, 'duration'))
                if duration:
                    SubElement(characteristics, 'Property',
                               attrib={'FormalName': 'TotalDuration', 'Value': str(duration)})

    def _format_package(self, article, main_news_component):
        """
        Constructs the NewsItemRef for packages.

        :param dict article:
        :param Element main_news_component:
        """
        news_component = SubElement(main_news_component, 'NewsComponent')
        SubElement(news_component, 'Role', {'FormalName': 'root'})
        for group in [group for group in article.get('groups', []) if group.get(GROUP_ID) != ROOT_GROUP]:
            sub_news_component = SubElement(news_component, 'NewsComponent')
            SubElement(sub_news_component, 'Role', attrib={'FormalName': group.get(ROLE, '')})
            for ref in group.get(REFS, []):
                if RESIDREF in ref:
                    revision = self._process_revision({})
                    item_ref = self._generate_public_identifier(ref.get(RESIDREF), ref.get(config.VERSION),
                                                                revision.get('Update', ''))
                    SubElement(sub_news_component, 'NewsItemRef', attrib={'NewsItem': item_ref})

    def _get_total_duration(self, duration):
        """
        Get the duration in seconds.

        :param str duration: Format required is '%H:%M:%S.%f'. For example, 1:01:12.0000 or 1:1:12.0000
        :return int:
        """
        total_seconds = 0
        try:
            ts = time.strptime(duration, '%H:%M:%S.%f')
            total_seconds = ts.tm_sec + (ts.tm_min * 60) + (ts.tm_hour * 60 * 60)
        except Exception:
            logger.error('Failed to get total seconds for duration:{}'.format(duration))
        return total_seconds

    def can_format(self, format_type, article):
        """
        Test if the article can be formatted to NewsML 1.2 or not.

        :param str format_type:
        :param dict article:
        :return: True if article can formatted else False
        """
        return format_type == 'newsml12' and \
            article[ITEM_TYPE] in {CONTENT_TYPE.TEXT, CONTENT_TYPE.PREFORMATTED, CONTENT_TYPE.COMPOSITE,
                                   CONTENT_TYPE.PICTURE, CONTENT_TYPE.VIDEO, CONTENT_TYPE.AUDIO}




# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2017 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import re
from lxml import etree  # noqa
from superdesk import etree as sd_etree
from lxml import html as lxml_html
from lxml.html import clean


# This pattern matches http(s) links, numbers (1.000.000 or 1,000,000 or 1 000 000), regulars words,
# compound words (e.g. "two-done") or abbreviation (e.g. D.C.)
WORD_PATTERN = re.compile(r'https?:[^ ]*|([0-9]+[,. ]?)+|([\w]\.)+|[\w][\w-]*')


def get_text_word_count(text):
    """Get word count for given plain text.

    :param str text: text string
    :return int: word count
    """
    return sum(1 for word in WORD_PATTERN.finditer(text))


def get_text(markup, content='xml', lf_on_block=False, space_on_elements=False):
    """Get plain text version of (X)HTML or other XML element

    if the markup can't be parsed, it will be returned unchanged
    :param str markup: string to convert to plain text
    :param str content: 'xml' or 'html', as in parse_html
    :param bool lf_on_block: if True, add a line feed on block elements' tail
    :param bool space_on_elements: if True, add a space on each element's tail
        mainly used to count words with non HTML markup
    :return str: plain text version of markup
    """
    try:
        root = sd_etree.parse_html(
            markup,
            content=content,
            lf_on_block=lf_on_block,
            space_on_elements=space_on_elements)
        text = etree.tostring(root, encoding='unicode', method='text')
        return text
    except etree.ParseError:
        return markup


def get_word_count(markup, no_html=False):
    """Get word count for given html.

    :param str markup: xhtml (or other xml) markup
    :param bool no_html: set to True if xml param is not (X)HTML
        if True, a space will be added after each element to separate words.
        This avoid to have construct like <hl2>word</hl2><p>another</p> (like in NITF)
        being counted as one word.
    :return int: count of words inside the text
    """
    if no_html:
        return get_text_word_count(get_text(markup, content='xml', space_on_elements=True))
    else:
        return get_text_word_count(get_text(markup, content='html', lf_on_block=True))


def get_char_count(html):
    """Get character count for given html.

    :param html: html string to count
    :return int: count of chars inside the text
    """
    return len(get_text(html))


def get_reading_time(word_count):
    """Get estimanted number of minutes to read a text

    Check https://dev.sourcefabric.org/browse/SDFID-118 for details
    :param int word_count: number of words in the text
    :return int: estimated number of minute to read the text
    """
    reading_time_float = word_count / 250
    reading_time_minutes = int(reading_time_float)
    reading_time_rem_sec = int((reading_time_float - reading_time_minutes) * 60)
    if reading_time_rem_sec >= 30:
        reading_time_minutes += 1
    return reading_time_minutes


def sanitize_html(html):
    """Sanitize HTML

    :param str html: unsafe HTML markup
    :return str: sanitized HTML
    """
    if not html:
        return ""

    blacklist = ["script", "style", "head"]

    root_elem = lxml_html.fromstring(html)
    cleaner = clean.Cleaner(
        add_nofollow=False,
        kill_tags=blacklist
    )
    cleaned_xhtml = cleaner.clean_html(root_elem)

    safe_html = etree.tostring(cleaned_xhtml, encoding="unicode")

    # the following code is legacy (pre-lxml)
    if safe_html == ", -":
        return ""

    return safe_html



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014, 2015 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import superdesk
from datetime import timedelta
from superdesk.utc import utcnow
from superdesk.utils import get_random_token


class CompanyTokenService(superdesk.Service):

    def create(self, docs, **kwargs):
        for doc in docs:
            doc['_id'] = get_random_token()
            doc.setdefault('expiry', utcnow() + timedelta(days=7))
        return super().create(docs, **kwargs)



from unittest.mock import Mock, call
from datetime import timedelta

from superdesk.tests import TestCase
from superdesk.utc import utcnow
from superdesk.utc import query_datetime

from .remove_exported_files import RemoveExportedFiles


class MockMediaFS:
    """Simple Mocked Media FS"""

    def __init__(self, files):
        self.files = files
        self.delete = Mock()
        self.delete.side_effect = self._delete
        self.find = Mock()
        self.find.side_effect = self._find

    def _delete(self, _id):
        del self.files[_id]

    def _find(self, folder=None, upload_date=None):
        files = []
        for file in self.files.values():
            if (upload_date is not None and not query_datetime(file['upload_date'], upload_date)) \
                    or not file['filename'].startswith(folder):
                continue
            files.append(file)
        return files


class RemoveExportedFilesTest(TestCase):

    def setUp(self):
        # Create a backup of the original media reference so we can set it back up again later
        self.original_media = self.app.media
        self.now = utcnow()
        self.app.media = MockMediaFS({
            'a1': {'_id': 'a1', 'filename': 'aa', 'upload_date': self.now},
            'b2': {'_id': 'b2', 'filename': 'bb', 'upload_date': self.now - timedelta(hours=20)},
            'c3': {'_id': 'c3', 'filename': 'cc', 'upload_date': self.now - timedelta(hours=25)},
            'd4': {'_id': 'd4', 'filename': 'dd', 'upload_date': self.now - timedelta(hours=36)},
            'e5': {'_id': 'e5', 'filename': 'temp/ee', 'upload_date': self.now},
            'f6': {'_id': 'f6', 'filename': 'temp/ff', 'upload_date': self.now - timedelta(hours=20)},
            'g7': {'_id': 'g7', 'filename': 'temp/gg', 'upload_date': self.now - timedelta(hours=25)},
            'h8': {'_id': 'h8', 'filename': 'temp/hh', 'upload_date': self.now - timedelta(hours=36)}
        })
        self.can_expire = ['g7', 'h8']

        self.command = RemoveExportedFiles()

    def tearDown(self):
        """Ensure to restore the original media reference"""
        self.app.media = self.original_media

    def test_get_file_ids(self):
        expire_at = utcnow() - timedelta(hours=24)
        for file_id in self.command._get_file_ids(expire_at):
            self.assertIn(file_id, self.can_expire)

        kwargs = {
            'folder': 'temp',
            'upload_date': {'$lte': expire_at}
        }
        self.app.media.find.assert_called_once_with(**kwargs)

    def test_remove_exported_files(self):
        expire_at = utcnow() - timedelta(hours=24)
        self.command._remove_exported_files(expire_at)

        for file_id in self.app.media.files:
            self.assertNotIn(file_id, self.can_expire)

        self.app.media.delete.assert_has_calls([call('g7'), call('h8')], any_order=True)
        self.assertEqual(self.app.media.delete.call_count, 2)

    def test_run(self):
        self.command.run()
        for file_id in self.app.media.files:
            self.assertNotIn(file_id, self.can_expire)

        self.app.media.delete.assert_has_calls([call('g7'), call('h8')], any_order=True)
        self.assertEqual(self.app.media.delete.call_count, 2)



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import os

import superdesk
from superdesk.io.commands.update_ingest import ingest_items
from .reuters_mock import setup_reuters_mock, teardown_reuters_mock


def setup_providers(context):
    app = context.app
    context.providers = {}
    context.ingest_items = ingest_items
    with app.test_request_context(app.config['URL_PREFIX']):
        rule_sets = {'name': 'reuters_rule_sets',
                     'rules': [
                         {"old": "@", "new": ""},
                     ]}

        result = superdesk.get_resource_service('rule_sets').post([rule_sets])

        app.config['REUTERS_USERNAME'] = 'no_username'
        app.config['REUTERS_PASSWORD'] = 'no_password'
        setup_reuters_mock(context)

        path_to_fixtures = os.path.join(os.path.abspath(os.path.dirname(__file__)), 'fixtures')
        providers = [
            {'name': 'reuters', 'source': 'reuters', 'feeding_service': 'reuters_http', 'feed_parser': 'newsml2',
             'is_closed': False, 'rule_set': result[0],
             'config': {'url': 'http://rmb.reuters.com/rmd/rest/xml',
                        'auth_url': 'https://commerce.reuters.com/rmd/rest/xml/login',
                        'username': app.config['REUTERS_USERNAME'], 'password': app.config['REUTERS_PASSWORD']}},
            {'name': 'AAP', 'source': 'AAP Ingest', 'feeding_service': 'file', 'feed_parser': 'nitf',
             'is_closed': False, 'critical_errors': {'2005': True}, 'config': {'path': path_to_fixtures}},
            {'name': 'teletype', 'source': 'AAP Teletype', 'feeding_service': 'file', 'feed_parser': 'zczc',
             'is_closed': False, 'config': {'path': path_to_fixtures}},
            {'name': 'DPA', 'source': 'DPA', 'feeding_service': 'file', 'feed_parser': 'dpa_iptc7901',
             'is_closed': False, 'config': {'path': path_to_fixtures}},
            {'name': 'AP', 'source': 'AP', 'feeding_service': 'file', 'feed_parser': 'ap_anpa1312',
             'is_closed': False, 'config': {'path': path_to_fixtures}},
            {'name': 'ninjs', 'source': 'NINJS', 'feeding_service': 'file', 'feed_parser': 'ninjs',
             'is_closed': False, 'config': {'path': path_to_fixtures}},
            {'name': 'email', 'source': 'Email', 'feeding_service': 'file', 'feed_parser': 'email_rfc822',
             'is_closed': False, 'config': {'path': path_to_fixtures, 'formatted': True}}
        ]

        result = superdesk.get_resource_service('ingest_providers').post(providers)
        context.providers['reuters'] = result[0]
        context.providers['aap'] = result[1]
        context.providers['teletype'] = result[2]
        context.providers['dpa'] = result[3]
        context.providers['ap'] = result[4]
        context.providers['email'] = result[5]


def teardown_providers(context):
    teardown_reuters_mock(context)




import os
import superdesk

from flask import current_app
from werkzeug.utils import secure_filename
from apps.auth import get_user_id


RESOURCE = 'attachments'


class AttachmentsResource(superdesk.Resource):
    schema = {
        'media': {'type': 'media'},
        'mimetype': {'type': 'string'},
        'filename': {'type': 'string'},
        'length': {'type': 'integer'},
        'title': {'type': 'string'},
        'description': {'type': 'string'},
        'user': superdesk.Resource.rel('users'),
    }

    item_methods = ['GET', 'PATCH']
    resource_methods = ['GET', 'POST']
    privileges = {'POST': 'archive', 'PATCH': 'archive'}


class AttachmentsService(superdesk.Service):
    def on_create(self, docs):
        for doc in docs:
            doc['user'] = get_user_id()
            if doc.get('media'):
                media = current_app.media.get(doc['media'], RESOURCE)
                doc.setdefault('filename', secure_filename(os.path.basename(getattr(media, 'filename'))))
                doc.setdefault('mimetype', getattr(media, 'content_type'))
                doc.setdefault('length', getattr(media, 'length'))

    def on_deleted(self, doc):
        current_app.media.delete(doc['media'], RESOURCE)


def init_app(app):
    superdesk.register_resource(RESOURCE, AttachmentsResource, AttachmentsService)
    app.client_config['attachments_max_files'] = app.config.get('ATTACHMENTS_MAX_FILES', 10)
    app.client_config['attachments_max_size'] = app.config.get('ATTACHMENTS_MAX_SIZE', 2 ** 20 * 8)  # 8MB



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license


from .roles import RolesResource, RolesService
import superdesk


def init_app(app):
    endpoint_name = 'roles'
    service = RolesService(endpoint_name, backend=superdesk.get_backend())
    RolesResource(endpoint_name, app=app, service=service)

    superdesk.privilege(name='roles', label='Roles Management', description='User can manage roles.')



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from superdesk import get_backend
from apps.languages.service import LanguagesService
from apps.languages.resource import LanguagesResource


def init_app(app):
    endpoint_name = 'languages'
    service = LanguagesService(endpoint_name, backend=get_backend())
    LanguagesResource(endpoint_name, app=app, service=service)

    languages = app.config.get('LANGUAGES', [])
    for language in languages:
        language['_id'] = language['language']



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from superdesk.resource import Resource
from content_api import MONGO_PREFIX


class UsersResource(Resource):
    """
    Users schema
    """

    schema = {
        'password': {
            'type': 'string',
            'minlength': 8
        },
        'first_name': {
            'type': 'string'
        },
        'last_name': {
            'type': 'string'
        },
        'email': {
            'unique': True,
            'type': 'string',
            'required': True
        },
        'phone': {
            'type': 'string',
            'nullable': True
        },
        'signup_details': {
            'type': 'dict'
        },
        'country': {
            'type': 'string'
        },
        'company': Resource.rel('companies', embeddable=True, required=False),
        'user_type': {
            'type': 'string',
            'allowed': ['administrator', 'internal', 'public'],
            'default': 'public'
        },
        'is_validated': {
            'type': 'boolean',
            'default': False
        },
        'is_enabled': {
            'type': 'boolean',
            'default': True
        },
        'is_approved': {
            'type': 'boolean',
            'default': False
        },
        'token': {
            'type': 'string',
        },
        'token_expiry_date': {
            'type': 'datetime',
        }
    }

    item_methods = ['GET', 'PATCH', 'PUT']
    resource_methods = ['GET', 'POST']
    mongo_prefix = MONGO_PREFIX
    datasource = {
        'source': 'users',
        'projection': {'password': 0},
        'default_sort': [('name', 1)]
    }
    mongo_indexes = {
        'email': ([('email', 1)], {'unique': True})
    }



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from .correct import CorrectPublishResource, CorrectPublishService  # NOQA
from .kill import KillPublishResource, KillPublishService  # NOQA
from .publish import ArchivePublishResource, ArchivePublishService  # NOQA
from .resend import ResendResource, ResendService  # NOQA



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license


import datetime
import os
import unittest

from superdesk.etree import etree
from superdesk.io.feed_parsers.wenn_parser import WENNFeedParser
from superdesk.utc import utc


class WENNTestCase(unittest.TestCase):
    filename = 'wenn.xml'

    def setUp(self):
        dirname = os.path.dirname(os.path.realpath(__file__))
        fixture = os.path.normpath(os.path.join(dirname, '../fixtures', self.filename))
        provider = {'name': 'Wenn'}
        with open(fixture, 'rb') as f:
            self.file = f.read()
            etree.fromstring(self.file)
            self.items = WENNFeedParser().parse(etree.fromstring(self.file), provider)

    def test_items_counts(self):
        self.assertEqual(len(self.items), 2)

    def test_headline(self):
        self.assertEqual(self.items[0].get('headline'), 'Suge Knight involved in fatal hit and run incident')
        self.assertEqual(self.items[1].get('headline'), 'Jagged Edge singer Kyle Norman arrested over domestic assault')

    def test_abstract(self):
        self.assertEqual(self.items[0].get('abstract'),
                         'Police officials in Los Angeles are investigating a fatal hit and run incident involving '
                         'rap mogul SUGE KNIGHT on Thursday (29Jan15).')
        self.assertEqual(self.items[1].get('abstract'),
                         'JAGGED EDGE singer KYLE NORMAN has been arrested by police in Atlanta, Georgia amid'
                         ' allegations of aggravated assault.')

    def test_anpa_category(self):
        self.assertEqual(self.items[0].get('anpa_category')[0]['qcode'], 'e')
        self.assertEqual(self.items[1].get('anpa_category')[0]['qcode'], 'e')

    def test_subject(self):
        self.assertEqual(self.items[0].get('subject')[0]['qcode'], '01000000')
        self.assertEqual(self.items[1].get('subject')[0]['qcode'], '01000000')

    def test_firstcreated(self):
        self.assertEqual(self.items[0].get('firstcreated'), datetime.datetime(year=2015, month=1, day=30, hour=0,
                                                                              minute=33, second=6, tzinfo=utc))

    def test_versioncreated(self):
        self.assertEqual(self.items[0].get('versioncreated'), datetime.datetime(year=2015, month=1, day=30, hour=2,
                                                                                minute=40, second=56, tzinfo=utc))

    def test_body(self):
        self.assertEqual(self.items[0].get('body_html'), '<p>This is body content1.</p>')
        self.assertEqual(self.items[1].get('body_html'), '<p>This is body content2.</p>')

    def test_item_defaults(self):
        self.assertEqual(self.items[0].get('pubstatus'), 'usable')
        self.assertEqual(self.items[0].get('urgency'), 5)
        self.assertEqual(self.items[0].get('type'), 'text')

    def test_guid(self):
        self.assertEqual(self.items[0].get('guid'), '1369426')
        self.assertEqual(self.items[1].get('guid'), '1369417')
